# consensus.pl
# Subroutine to build consensus view.
sub draw_consensus {

# Must set so that we can add text to the windows.
$text_v->configure(-state => 'normal');
$llabel->configure(-state => 'normal');
$rlabel->configure(-state => 'normal');
# Remove anything we used to have in the windows.
$text_v->delete('1.0', 'end');
$llabel->delete('1.0', 'end');
$rlabel->delete('1.0', 'end');

# Set of gradient backgrounds used in conjunction with the
# certainty of this letter.  (Higher probability is assumed
# to be higher certainty.  This breaks somewhat if have two
# letters of equal high value).
$text_v->tagConfigure("c0", -background => 'grey97');
$text_v->tagConfigure("c1", -background => 'grey91');
$text_v->tagConfigure("c2", -background => 'grey85');
$text_v->tagConfigure("c3", -background => 'grey79');
$text_v->tagConfigure("c4", -background => 'grey73');
$llabel->tagConfigure("label", -background => 'white');
$rlabel->tagConfigure("label", -background => 'white');
$text_v->tagConfigure("scroll", -foreground => 'magenta',
	-borderwidth => 1, -font => $courierbold,
	-underline => 1);

# Consensus View and other windows need to be aware of window
# resizing and fit text accordingly.  I believe reqwidth pg 279
# might work.

# Fill consensus view with consensus sequence tagged
# for click scrolling.
$llabel->insert('end', sprintf("%4d", 1), ["label"]);
for ($s = 1; $s <= $leng; $s++) {
  $text_v->tagConfigure("t$s");   # Tag unique to this letter
  # Set edges of window centered around current character,
  # unless character is within current viewing number of edge.
     # Bind that unique tag to a function call.
  $text_v->tagBind("t$s", "<Button-1>", [\&scroll_by_consensus, $s]);
  $charprob = $match[$s][$maxchar_m[$s]]; # Determine currchar probability

  # As lower probabilites are more likely this is ordered to minimize
  # the if traversals.
 if ($charprob < .2) {
        $text_v->insert('end', $E_alpha_char[$maxchar_m[$s]], ["c0", "t$s"]);
  } elsif ($charprob < .4) {
        $text_v->insert('end', $E_alpha_char[$maxchar_m[$s]], ["c1", "t$s"]);
  } elsif ($charprob < .6) {
        $text_v->insert('end', $E_alpha_char[$maxchar_m[$s]], ["c2", "t$s"]);
  } elsif ($charprob < .8) {
        $text_v->insert('end', $E_alpha_char[$maxchar_m[$s]], ["c3", "t$s"]);
  } else {
        $text_v->insert('end', $E_alpha_char[$maxchar_m[$s]], ["c4", "t$s"]);
  }

  if ($s%40 == 0) {
    $text_v->insert('end', "\n");       # Set Line Boundries
    $text_v->update();                  # Flush Buffer
    $rlabel->insert('end', sprintf("%4d\n", $s), ["label"]);
    $llabel->insert('end', sprintf("\n%4d", ($s+1)), ["label"]);
  } elsif ($s%10 == 0) {
    $text_v->insert('end', " ");   #Set word boundries
    $text_v->update();          # Flush buffer
  }
}
$rlabel->insert('end', sprintf("%4d", $s));

# Prevents user insert into text box.  Must be done last because
# also prevents additional inserts within program.
$text_v->configure(-state => 'disabled');
$llabel->configure(-state => 'disabled');
$rlabel->configure(-state => 'disabled');

$tmp = $nodes -1;
$text_v->tagAdd("scroll", '1.0', "1.$tmp");
}

sub update_consensus {
  &scroll_by_consensus($text_v, $center_value);
}

$end = 1;
