#  Right Hand Histograms

sub draw_hist {
  my ($state_i, $xcord) = @_;	

$instate = 1;

# Constants for histogram sizing.
$h_width = 2*$interval/$alpha;
$th_width = $interval/12;
$in_p7 = 0;

#############
# Clean any old data from views
#
$mprob_v->delete("histogram");
$iprob_v->delete("histogram");
$tprob_v->delete("histogram");
$prob_title->delete("histogram");
$mmodel_v->delete("histogram");

###############
# Highlight box in main model view
#
$mmodel_v->createLine($xcord, 0, $xcord, 3*$interval,
	-fill => $highlight_color,
	-width => 3,
	-tags => ['histogram', 'box']);
$mmodel_v->createLine($xcord,0,$xcord+$node_width,0,
	-fill => $highlight_color,
	-width => 3,
	-tags => ['histogram', 'box']);
$mmodel_v->createLine($xcord + $node_width, 0, 
	$xcord + $node_width, 3*$interval,  
	-fill => $highlight_color,
	-width => 3,
	-tags => ['histogram', 'box']);
$mmodel_v->createLine($xcord,3*$interval,
	$xcord+$node_width,3*$interval,
	-width => 5,
	-fill => $highlight_color,
	-tags => ['histogram', 'box']);

################
# Build histograms
#
for ($k = 0; $k < $alpha; $k++) {
  if ($use_probs) {
    $x1 = 0;
    $x2m = 1.5*$interval*$match[$state_i][$k];
    $x2i = 1.5*$interval*$insert[$state_i][$k]; 
  } else {
    $x1 = 0.75*$interval;
    if ($match_score[$state_i][$k] > 0) {  # Positive Score
       $x2m = $x1 + 0.75*$interval*($match_score[$state_i][$k]/5000);
    } else {				   # Negative Score
       $x2m = $x1 - 0.75*$interval*($match_score[$state_i][$k]/(-5000));
    }
    if ($insert_score[$state_i][$k] > 0) {  # Positive Score
       $x2i = $x1 + 0.75*$interval*($insert_score[$state_i][$k]/5000);
    } else {				   # Negative Score
       $x2i = $x1 - 0.75*$interval*($insert_score[$state_i][$k]/(-5000));
    }
  }
# Match Histogram
  $mprob_v->createRectangle(
	$x1,$k*$h_width,
	$x2m, $k*$h_width + $h_width,
	-fill => $match_color,
	-tags => 'histogram',
	-outline => 'black');
  $mprob_v->createText(
	2*$interval, $k*$h_width + $h_width,
	-anchor => 'se',
	-fill => 'black',
	-font => $courier2,
	-tags => 'histogram',
	-text => &print_score('M', $state_i, $k));
# Insert Histogram
  $iprob_v->createRectangle(
	$x1,$k*$h_width,
	$x2i, $k*$h_width + $h_width,
	-fill => $insert_color,
	-tags => 'histogram',
	-outline => 'black');
  $iprob_v->createText(
	2*$interval,$k*$h_width +$h_width,
	-anchor => 'se',
	-fill => 'black',
	-font => $courier2,
	-tags => 'histogram',
	-text => &print_score('I', $state_i, $k));
}

# This for loop maps the indicies of the transition array ($s)
# as read from an HMMER file into groups which sum to 1.
# They are output in 12 bars (0-11) indexed by $k

for ($k = 0, $s = 0; $k < 12; $k++) {
# Transition Histogram
  if ($k == 3) { $s = 8;	# The fourth bar (3) should be m->e (8)
    &print_histogram($k, $state_i, $s);
  }
  elsif ($k == 4) { }		# Leave the fifth bar (6) blank 
  elsif ($k == 5) { $s = 3;	# The sixth bar (5) should be i->m (3)
    &print_histogram($k, $state_i, $s);
  }
  elsif ($k == 7) { }		# Leave the eighth (7) bar blank
  elsif ($k == 8) { $s = 5;	# The ninth (8) bar should be d->m (5)
    &print_histogram($k, $state_i, $s);
  }
  elsif ($k == 10) { }		# Leave the eleventh (10) bar blank
  elsif ($k == 11) { $s = 7;	# The twelth (11) bar should be b->m (7)
    &print_histogram($k, $state_i, $s);
  } 
  else {
    &print_histogram($k, $state_i, $s);
  }
  $s++;
}
$prob_title->createText(0.3*$interval, 2,
	-anchor => 'n',
	-tags => 'histogram',
	-text => $state_i);
	
return 1;
}

# Prints the transition histogram
sub print_histogram {
  my $k = shift;	# Current bar number counting from top of histogram
  my $st = shift;	# Current main model node number
  my $s = shift;	# Current index into transition array

  $y1 = $k*$interval/12;
  $y2 = $y1 + $interval/12; 

  # This function determines the length of the bar
  if ($use_probs) {
    $x1 = 0;
    $x2 = 1.5*$interval*$transition[$st][$s];
  } else {
    $x1 = 0.75*$interval;
    if ($trans_score[$st][$s] > 0) {  # Positive Score
       $x2 = $x1 + 0.75*$interval*($trans_score[$st][$s]/5000);
    } else {				   # Negative Score
       $x2 = $x1 - 0.75*$interval*($trans_score[$st][$s]/(-5000));
    }
  }

  $tprob_v->createRectangle($x1,$y1,$x2,$y2, 
        -fill => $transition_color,
        -tags => 'histogram',
        -outline => 'black');
  $tprob_v->createText(
        2*$interval,$y2,
        -anchor => 'se',
        -fill => 'black',
        -font => $courier2,
        -tags => 'histogram',
        -text => &print_score('T', $st, $s));
  return 1;
}

# Routine to format labels (scores/probs) for histogram view
sub print_score {
  my $type = shift;
  my $st = shift;
  my $pos = shift;

  if ($type eq 'M') {
    if ($use_probs) {
       return sprintf "$E_alpha_char[$pos]: %0.${digits}f", $match[$st][$pos];
    } else {
       return sprintf "$E_alpha_char[$pos]: %6d", int($match_score[$st][$pos]);
    }
  } elsif($type eq 'I') {
    if ($use_probs) {
       return sprintf "$E_alpha_char[$pos]: %0.${digits}f", $insert[$st][$pos];
    } else {
       return sprintf "$E_alpha_char[$pos]: %6d", int($insert_score[$st][$pos]);
    }
  } else {
    if ($use_probs) {
 	return sprintf "$trans_names[$pos]: %0.${digits}f", $transition[$st][$pos];
    } else {
	return sprintf "$trans_names[$pos]: %6d", int($trans_score[$st][$pos]);
    }
  }
}

$end = 1;
