# hmmfunc.pl
# subroutines necessary to convert an *.hmm file
# into its probability format.

my $LBASE2 = 1.44269504;
my $EBASE2 = 0.69314718;
my $INTSCALE = 1000;
my $INFTY = 987654321;

sub Prob2Score {	#Convert a probability to a score (output)
  my $prob = shift;
  my $nul_model = shift;

  if ($prob == 0) {
    return (int(-1*$INFTY));
  } else {
    return (int(0.5 + $INTSCALE*&sreLOG2($prob/$nul_model)));
  }
}
sub Score2Prob {	#Convert a score (output) to a probability
  my $score = shift;
  my $nul_model = shift;
  if ($score eq int(-1*$INFTY)) {
    return 0;
  } else {
    return ($nul_model*&sreEXP2($score/$INTSCALE));
  }
}
sub Ascii2Prob {	#Convert input values (* & nums) to Probs
  my $ascii = shift;
  my $nul_model = shift;
  if ($ascii eq "*") {
     return 0;
  } else {
     return &Score2Prob($ascii, $nul_model);
  } 
}
sub Prob2Ascii {	#Converts probs to output format
  my ($prob, $nul_model) = shift;
  if ($prob == 0) {
    return "*";
  } else {
    return (&Prob2Score($prob, $nul_model));
  }
}
sub sreLOG2 {		#Log base 2 as used in HMMer
  my $x = shift;
  if ($x > 0) {
    return (log($x)*$LBASE2);
  } else {
    return (-9999.0);
  } 
}
sub sreEXP2 {		#2^(value) as used in HMMer
  my $x = shift;
  return (exp($x*$EBASE2));
}

