# hmmparse.pl
# Read in a *.hmm file and build appropriate data structures
# in perl.

require("hmmfunc.pl");

sub LoadHMM {
  my $fn = shift;

  $i = 0;

  open HMM, $fn || die "Could not find HMM file!";

  while (<HMM>) {
    if ($_ =~ /^NAME\s*(\S+).*/) {
      $models[$i] = $1;
      $i++;
    }
  }

  close (HMM);

  if (scalar(@models) > 1) {
    $selector = $mw->Toplevel();
    $selector->title("Choose a Model");
    $f = $selector->Frame->pack(-side => 'top', -fill => 'x');
    $f->Label(-text => "Filename: $filename")
        ->pack(-side => 'top', -anchor => 'w');
    $f->Button(-text => "Select", -command =>
       sub{$iter = &get_file($fn);})->pack(-side=>'top');
    $t = $selector->Scrolled("Listbox", -scrollbars => "oe")
        ->pack(-side => "bottom");
    $t->configure(-selectmode => "single");
    $t->insert("end", sort(@models));
    $selector->grab();
  } else {
    &parse_file($fn, 0);
  }
} # End LoadHMM function

sub parse_file {
  my $fn = shift;
  my $iter = shift;

  # Boolean variables for proper parsing of *.hmm
  my $in_model = 0;
  my $got_trans = 0;
  my $in_state = 0;
  my $in_record = 0;

  $statenum = 0;	#Array Index

  open HMM, $fn || die "Could not find HMM file!";

  $cnt = 0;
  while(<HMM>) {
   if (!($in_record)) { 
     if ($_ =~/^HMMER(\S+)/) {
	if ($cnt == $iter) {	# Want this model
            push @out_file, "\nHMMER2.0\n";	
	    $version = $1;
            $in_record = 1;
        }
        $cnt++; 
     }
   } else {
    # Parse the *.hmm record
    push @out_file, $_;	# Keep full file in line array for future use.

    if ($_ =~ /^NAME\s*(\S+).*/) { $model_name = $1; }
    elsif ($_ =~ /^LENG\s*(\S+).*/) { $leng = $1; }
    elsif ($_ =~ /^ALPH\s*(\S+).*/) { 
	if ($1 eq "Amino") { $alpha = 20; } else { $alpha = 4; }
    }
    elsif ($_ =~ /^XT\s*(.+)/) {
      @xt_s = split (" ", $1);
      for ($i = 0; $i < @xt_s; $i++) {
         $xt[$i] = &Ascii2Prob($xt_s[$i], 1);
      }
    }
    elsif ($_ =~ /^NULT\s*(.+)/) {
      @nult = split (" ", $1);
      for ($i = 0; $i < @nult; $i++) {
	 $nult[$i] = &Ascii2Prob($nult[$i], 1);
      }
    }
    elsif ($_ =~ /^NULE\s*(.+)/) {
      @nule = split (" ", $1);
      for ($i = 0; $i < @nule; $i++) {
	 $nule[$i] = &Ascii2Prob($nule[$i], 1/$alpha);
      }
    }
    elsif ($_ =~ /^\/\//) {
      last;
    }
    # In the HMM model section 
    elsif ($_ =~ /^HMM\s*(.+)/) {
       @E_alpha_char = split(" ", $1);
       $in_model = 1;
    }
    # The human readable line which is the transition names
    elsif (!$got_trans && $in_model && ($_ =~ /\s*(.+)/)) {
       @trans_names = split (" ", $1);
       $got_trans = 1;
    }
    # The special B->D transition line
    elsif (!$in_state && $got_trans && ($_ =~ /\s*(.+)/)) {
       @edge_t= split(" ", $1);
       for ($i = 0; $i < @edge_t; $i++) {
         $edge_t[$i] = &Ascii2Prob($edge_t[$i], 1.0);
       }
       $in_state = 1;
    }
    elsif ($in_state && ($_ =~ /\s*(\S+)\s*(.+)/)) {
       @temp_array = split(" ", $2);
    # Transition Line 
     if (($1 eq '-') && (scalar(@temp_array) == 9)) {
	$trans_score[$statenum] = [@temp_array];
	for ($i = 0; $i < @temp_array; $i++) {
	   $temp_array[$i] = &Ascii2Prob($temp_array[$i], 1);
        }
	$transition[$statenum] = [@temp_array];
    # Insert Line
     } elsif ($1 eq '-') {
	$insert_score[$statenum] = [@temp_array];
	for ($i = 0; $i < @temp_array; $i++) {
	   $temp_array[$i] = &Ascii2Prob($temp_array[$i], $nule[$i]);
        }
	$insert[$statenum] = [@temp_array];
    # Match Line
     } else {
	$statenum = int($1);
	$match_score[$statenum] = [@temp_array];
	for ($i = 0, $max_index = 0; $i < @temp_array; $i++) {
	   $temp_array[$i] = &Ascii2Prob($temp_array[$i], $nule[$i]);
	   if ($temp_array[$i] > $temp_array[$max_index]) {
		$max_index = $i;
	   }
        }
	$match[$statenum] = [@temp_array];
	$maxchar_m[$statenum] = $max_index;
     } # match
   } # in_state
  } # else
 } # End while loop

 close HMM || die "Improper closing of HMM file.";
 &draw_consensus();
 &build_mmodel_v();
 &draw_plan7_v();
 &scroll_by_consensus($text_v, 1);
 $mw->configure(-cursor => "arrow");
 $text_v->configure(-cursor => "arrow");
 $llabel->configure(-cursor => "arrow");
 $rlabel->configure(-cursor => "arrow");
 $file_b->grabRelease();

} # End parse_file subroutine


sub get_file {
  my $fn = shift;

  $fsi = $t->curselection;      # Get index of selection
  $fs = $t->get($fsi);          # Convert index to element

  $index = 0;

  #Find element value in @models
  for ($i=0; $i < @models; $i++) {
     if ($models[$i] == $fs) {
        $index = $i;
        $i = scalar(@models) + 1;
     }
  }
  $selector->grabRelease();
  $selector->withdraw;
  $mw->update();
  &parse_file($fn, $index);
}


