#Build menu buttons and contents

require("FileDialog.pm");  # Version 1.2 by Brent B. Powers

$LoadDialog = $menu->FileDialog(-Title => 'Load a HMM Model',
		-Create => 0);
$LoadDialog->configure(-FPat => '*hmm', -Path => $path,
	-ShowAll => 'NO', -DisableShowAll => 1);

$file_b = $menu->Menubutton(-text => "File",
	-underline => 0,
        -menuitems => [['command' => "Load HMM model", 
		-underline => 0,
		-command => sub{
			$mmodel_v->configure(-takefocus => 0);
			$filename= $LoadDialog->Show(-Horiz => 1);
			if (defined($filename)) {
				&load_hmmfile;
			}
			$mmodel_v->configure(-takefocus => 1);
		}],
		['command' => "View model file", -underline => 5,
		-command => \&view_hmm],
		['command' => "Exit ", -underline => 1,
		-command => sub{exit}]]
        )->pack(-side => 'left', -padx => 4);
$edit_b = $menu->Menubutton(-text => "View",
	-underline => 0,
        -disabledforeground => 'grey',
        -menuitems => [ ['command' => "Zoom In", -underline => 5,
		-command => \&Zoom_In],
		['command' => "Zoom Out", -underline => 5,
		-command => \&Zoom_Out],
		['command' => "Hide Histogram View", -underline => 0,
		-command => \&Collapse]]
        )->pack(-side => 'left', -padx => 4);
$opt_b = $menu->Menubutton(-text => "Options",
	-underline => 1,)
	->pack(-side => 'left', -anchor => 'n');

# Checkable options in options menu.
$opt_b->checkbutton(-label => "Show Insert Histograms", 
		-variable => \$show_i_hist,
		-command => \&control_ihist);
$opt_b->checkbutton(-label => "Show Match Histograms",
	-variable => \$show_m_hist,
	-command => \&control_mhist);

$file_b->menu()->configure(-postcommand => \&update_menu);
$edit_b->menu()->configure(-postcommand => \&update_menu);
$opt_b->menu()->configure(-postcommand => \&update_menu);

#####################
# Menu Subroutines
#
sub update_menu {
  if ($filename ne "") {	# Have a loaded file
    if (!($nodes > 3)) {		# At min zoom level (3 nodes per screen)
      $edit_b->menu->entryconfigure(1, -state => "disabled");
      $edit_b->menu->entryconfigure(2, -state => "normal");
      $edit_b->menu->entryconfigure(3, -state => "normal");
    } elsif (!($nodes < 17)) {	# At max zoom level (17 nodes per screen)
     $edit_b->menu->entryconfigure(1, -state => "normal");
     $edit_b->menu->entryconfigure(2, -state => "disabled");
     $edit_b->menu->entryconfigure(3, -state => "normal");
    } else {
      $edit_b->menu->entryconfigure(1, -state => "normal");
      $edit_b->menu->entryconfigure(2, -state => "normal");
      $edit_b->menu->entryconfigure(3, -state => "normal");
    }

    if ($nodes > 9) {	# No show insert Histogram
     $opt_b->menu->entryconfigure(1, -state => "disabled");
    } else {
     $opt_b->menu->entryconfigure(1, -state => "normal");
    }
    $file_b->menu->entryconfigure(2, -state => "normal");
    $opt_b->menu->entryconfigure(2, -state => "normal");
  } else {			# Currently no loaded file
     $file_b->menu->entryconfigure(2, -state => "disabled");
     $edit_b->menu->entryconfigure(1, -state => "disabled");
     $edit_b->menu->entryconfigure(2, -state => "disabled");
     $edit_b->menu->entryconfigure(3, -state => "disabled");
     $opt_b->menu->entryconfigure(1, -state => "disabled");
     $opt_b->menu->entryconfigure(2, -state => "disabled");
  }
}

sub Zoom_In {
 if ($nodes > 3) {
  $mw->configure(-cursor => "watch");
  $text_v->configure(-cursor=> "watch");
  $llabel->configure(-cursor=> "watch");
  $rlabel->configure(-cursor=> "watch");
  $menu->grab();
  $tmp = $center_value;
  @list = $text_v->tagRanges("scroll");
  if ($list[0] != "") {
     $text_v->tagRemove("scroll", $list[0], $list[1]);
  }
  $mmodel_v->delete("all");
  $mmodel_v->update();

  $nodes = $nodes - 2;
  &set_scaling();
  &build_mmodel_v();
  $center_value = $tmp;
  $tmp = $nodes -1;
  $text_v->tagAdd("scroll", '1.0', "1.$tmp");
  &update_consensus();
  if ($curr_select > 0) {
    &draw_hist($curr_select, ($curr_select - $start_value)*$node_width);
  }
  $mw->configure(-cursor => "arrow");
  $text_v->configure(-cursor=> "arrow");
  $llabel->configure(-cursor=> "arrow");
  $rlabel->configure(-cursor=> "arrow");
  $menu->grabRelease();
 } # Doesn't allow zooming in closer than 3 nodes per screen
}

sub Zoom_Out {
 if ($nodes < 17) {
  $mw->configure(-cursor => "watch");
  $text_v->configure(-cursor=> "watch");
  $llabel->configure(-cursor=> "watch");
  $rlabel->configure(-cursor=> "watch");
  $menu->grab();
  $tmp = $center_value;
  @list = $text_v->tagRanges("scroll");
  if ($list[0] != "") {
     $text_v->tagRemove("scroll", $list[0], $list[1]);
  }
  $mmodel_v->delete("all");
  $mmodel_v->update();

  $nodes = $nodes + 2;
  &set_scaling();
  &build_mmodel_v();
  $center_value = $tmp;
  $tmp = $nodes -1;
  $text_v->tagAdd("scroll", '1.0', "1.$tmp");
  &update_consensus();
  if ($curr_select > 0) {
    &draw_hist($curr_select, ($curr_select - $start_value)*$node_width);
  }
  $mw->configure(-cursor => "arrow");
  $text_v->configure(-cursor=> "arrow");
  $llabel->configure(-cursor=> "arrow");
  $rlabel->configure(-cursor=> "arrow");
  $menu->grabRelease();
 } # Doesn't allow zooming out further than 19 nodes per screen
}

sub control_ihist {
  $main_v->update();
  if ($show_i_hist == 0) {
    $mw->configure(-cursor => "watch");
    $mmodel_v->delete("ihist");
    $mw->configure(-cursor => "arrow");
  } else {
    &mm_insert_hist();
  }
}
sub control_mhist {
  $main_v->update();
  if ($show_m_hist == 0) {
    $mmodel_v->delete("mhist");
  } else {
    $mw->configure(-cursor => "watch");
    &mm_match_hist();
    $mw->configure(-cursor => "arrow");
  }
}

sub load_hmmfile {
  # Clean up old model to show doing something
  $mw->title("HMMview v1.0 $filename");
  $mw->configure(-cursor => "watch"); 
  $file_b->grab();
  $text_v->configure(-cursor => "watch");
  $llabel->configure(-cursor => "watch");
  $rlabel->configure(-cursor => "watch");
  if ($hist_visible != 0) {
    &Collapse();
  }
  $text_v->configure(-state => 'normal');
  $text_v->delete('1.0', 'end'); 
  $llabel->delete('1.0', 'end');
  $rlabel->delete('1.0', 'end');
  $text_v->update();
  $llabel->update();
  $rlabel->update();
  $mmodel_v->delete("all");
  $mmodel_v->update();
  $mw->update();

  $plan7_v->delete("scrollline", "title", "p7sb");
  $p7scroll->delete("p7sb");
  $plan7_v->update();
 
  $nodes = 9;     # Reset zoom level
  &set_scaling();

  # Clean up old data values 
  @models = [];
  @out_file = [];

  &LoadHMM($filename);

}

sub view_hmm {

$textbox = $mw->Toplevel();
$textbox->title("Text Viewer: $filename");
$f = $textbox->Frame->pack(-side => 'top', -fill => 'x');
$f->Label(-text => "Filename: $filename")->pack(-side => 'left', 
        -anchor => 'w');
$close_b = $f->Button(-text => "Close Text Viewer", 
	-underline => 0,
	-command => sub {$close_b->grabRelease(); $textbox->withdraw;},
	)-> pack(-side=>'right');
$t = $textbox->Scrolled('Text', -scrollbars => 'ow')->
	pack(-side => 'bottom', -fill => 'both', -expand => 1);

if (!open(FH, "$filename")) {
  $t->insert("end", "ERROR: Could not open $filename\n");
        return;
}
for ($i=1; $i < @out_file; $i++) { 
  $t->insert("end", $out_file[$i]);
}
close (FH);
$t->configure(-state => 'disabled');

}

$end = 1;
