# mw.pl
# Main Window and frames structure.

# Main Window declaration
$mw = MainWindow->new;
$mw->title("HMMview v1.0 $filename");

$menu = $mw->Frame(
	-borderwidth => 0,
	)->pack(-side => 'top',
	   -fill => 'x');

# Split Main Window into main and histogram frames.
$main_v = $mw->Frame(
        -borderwidth => 0,
        )->pack(-fill => 'both', -expand => 1,
	  -side => 'left');

# Consensus View
# Contains three components: Left Label, Main Text, and Right Label
# Left & Right Labels are for numerical tags indicating location.
# Needs to be scrollable if text exceeds the window.
$con_v = $main_v->Frame(-borderwidth=> 0, -takefocus => 0,
	)->pack(-fill => 'both', -expand => 1, -side => 'top');
$cscroll = $con_v->Scrollbar(-takefocus => 0);
$llabel = $con_v->Text( -background => 'white',
        -highlightcolor => 'white', -selectbackground => 'white',
        -font => $courier, -padx => 5, -pady => 8,
        -wrap => 'char', -takefocus => 0,
	-spacing1 => 3, -spacing2 => 3, -spacing3=> 5,
        -height => 5, -width => 5);
$text_v = $con_v->Text( -background => 'white',
        -highlightcolor => 'white', -selectbackground => 'white',
        -font => $courier, -padx => 5, -pady => 8,
	-selectborderwidth => '1', -selectforeground => 'magenta',
        -wrap => 'word', -takefocus => 0,
	-spacing1 => 3, -spacing2 => 3, -spacing3=> 5,
        -height => 5, -width => 45);
$rlabel = $con_v->Text( -background => 'white',
        -highlightcolor => 'white', -selectbackground => 'white',
        -font => $courier, -padx => 5, -pady => 8,
        -wrap => 'char', -takefocus => 0,
	-spacing1 => 3, -spacing2 => 3, -spacing3=> 5,
        -height => 5, -width => 5);

# Tk throws an error on exiting the application is these are
# put into the constructors above.  Consequently they are
# configured outside the constructor to elimiate the error.
$text_v->configure(-cursor => "arrow");
$llabel->configure(-cursor => "arrow");
$rlabel->configure(-cursor => "arrow");

# Array of the three text windows
$texts = [$llabel, $text_v, $rlabel];
# Configure each text window to call &scroll_consensus
foreach $box (@$texts) {
 $box->configure(-yscrollcommand => [\&scroll_consensus, $cscroll,
                                        $box, $texts]);
}

# Configure the scrollbar to scroll each text window.
$cscroll->configure(-command => sub {foreach $box (@$texts) {
                                     $box->yview(@_);
                                     }});

# Pack all the text windows and scrollbar into the frame.
$llabel->pack(-anchor => 'nw',
          -padx => 0, -pady => 0,
          -side => 'left', -fill => 'y',
          -expand => 0);
$text_v->pack(-side => 'left', -fill => 'both',
          -expand => 1);
$rlabel->pack(-anchor => 'ne',
          -padx => 0, -pady => 0,
          -side => 'left', -fill => 'y',
          -expand => 0);
$cscroll->pack(-side => 'left', -fill => 'y');

#############
# Main Model View
#
# ScrollBar - linked to subroutine for scrolling multiple widgets
# in multiple windows.

#$scroll = $main_v->Scrollbar(-orient => 'horizontal',
#	-troughcolor => 'grey', -takefocus => 0, 
#	-background => 'white');
$mmodel_v = $main_v->Canvas( -background => 'white',
        -height => 3.2*$interval,   # The number is to prevent size
					# shifting when hist opens
#	-xscrollcommand => [\&scrolling, $scroll],
#	-scrollregion => [0,0, ($nodes+4)*$node_width, 3*$interval + 17],
#	-xscrollincrement => $node_width,
        -width => 4*$interval,  
	-takefocus => 1,
        )->pack(-side => 'top',
          -fill => 'both',
          -expand => 1);
# Need to configure scrollbar to correspond to mmodel_v
#$scroll->pack(-fill => 'x');

# Alternatively I can build my own scrollbar:
$scrolbar = $main_v->Frame(-background => 'white',
	)->pack(-side => 'top');
$sc_left = $scrolbar->Button(-anchor => 'center',
	-background => 'lightgrey', 
	-command => [\&shift_mm, -1],
	-text => "<=",
	)->pack(-fill => 'y', -side => 'left', -anchor => 'e');
$sc_right = $scrolbar->Button(-anchor => 'center',
	-background => 'lightgrey',
	-command => [\&shift_mm, +1],
	-text => "=>",
	)->pack(-fill => 'y', -side => 'right', anchor => 'n');
$filler = $scrolbar->Canvas(-background => 'white',
	-borderwidth => 0, -highlightbackground => 'white',
	-width => 3.4*$interval, -height => 10, -highlightcolor => 'white',
	)->pack(-expand => 'x', -fill => 'x');

############
# Plan Seven View
#
$plan7_v = $main_v->Canvas(-background => 'white',
	-border => 0,
        -height => $interval,
	-takefocus => 0,
        -width => 4*$interval,
        )->pack(-side => 'top',
          -fill => 'both',
          -expand => 1);

sub shift_mm {
  my $val = shift;

  $shiftval = $center_value + $val*($nodes-1);
  &scroll_by_consensus($sc_right, $shiftval);
}

##########
# Histogram View 
#
sub build_hist_v {
  $hist_v = $mw->Frame(
 	 -borderwidth => 0,
        )->pack(
          -side => 'right',
	  -fill => 'both',
          -expand => 1);
  $title_line = $hist_v->Frame(-borderwidth => 0
	)->pack(-side => 'top', -fill => 'both');
  $off_b = $title_line->Button(-text => "<-",
	-command => \&Collapse)->pack(
	-side => 'left');
  $prob_title = $title_line->Canvas(-background => 'white',
 	-width => 0.7*$interval,
	-height => 20, -takefocus => 0, 
	)->pack(-side => 'left', -fill => 'y');
  $score_b = $title_line->Radiobutton(-text => "Scores",
	-value => 0,
	-variable => \$use_probs,
	-command => \&update_hist, 
	)->pack(-side => 'right');
  $prob_b = $title_line->Radiobutton(-text => "Probs",
	-value => 1,
	-variable => \$use_probs,
	-command => \&update_hist,
	)->pack(-side => 'right');
  $prob_b->select();
  $tprob_v = $hist_v->Canvas(-background =>'white',
        -confine => 0,
        -height => $interval,
        -width => 2*$interval,
	-takefocus => 0,
        )->pack(-side => 'bottom',
         -fill => 'both',
          -expand => 1);
  $iprob_v = $hist_v->Canvas(-background =>'white',
        -confine => 0,
        -height => 2*$interval,
        -width => 2*$interval,
	-takefocus => 0,
        )->pack(-side => 'bottom',
          -fill => 'both',
          -expand => 1);
  $mprob_v = $hist_v->Canvas(-background =>'white',
        -confine => 0,
        -height => 2*$interval,
        -width => 2*$interval,
	-takefocus => 0,
        )->pack(-side => 'bottom',
          -fill => 'both',
          -expand => 1);
}

##############
# Scroll Routines
#

sub scroll_consensus {
  my ($sb, $scrolled, $lbs, @args) = @_;
  $sb->set(@args);      #Tell scrollbar what to display
}

# Subroutine which handles scrolling changes which occur when
# a letter in the Consensus view is clicked.
sub scroll_by_consensus {
   my ($txt, $n) = @_;

   $spacer = int($nodes/2);	# Centering guide

   $leftend = ($n - $spacer);
   $rightend = ($n + $spacer);
 
   # Boundry checks. 
   if ($leftend < 1) {
	$leftend = 1;
	$rightend = $nodes;
   }
   if ($rightend > $leng) {
	$leftend = $leng - $nodes + 1;
	$rightend = $leng;
   }
   $center_value = $leftend + $spacer;

   $arg[0] = $leftend/($leng+2);
   $arg[1] = $rightend/($leng+2);

   $start_value = $leftend;
   &build_trans();

  if ($hist_visible) {	# Currently Hist view is active
     if (($curr_select >= $start_value) && 
	($curr_select <= ($start_value + $nodes))) {
        &draw_hist ($curr_select, ($curr_select - $start_value)*$node_width);
     } else {
        $mmodel_v->delete("histogram");
     }
  }

  # Set new scroll boxes
  if ($filename ne "") {
   if ($leftend <= 0) {
     $left_s = "1.0";
   } else {
 # Would this be better if it were: 	??
 # $left_s = $text_v->index("t$leftend".".first");
     $left_s = $text_v->tagRanges("t$leftend");
     $left_s = "$left_s - 1 chars";
   }
   if ($rightend > $leng) {
     $right_s = $text_v->index('end');
   } else {
     $right_s = $text_v->tagRanges("t$rightend");
   }
   # Remove old highlights
   @list = $text_v->tagRanges("scroll");
   if ($list[0] != "") {
    $text_v->tagRemove("scroll", $list[0], $list[1]);
    $text_v->tagAdd("scroll", "$left_s", "$right_s");
   }   
   $plan7_v->delete("scrollline");
   $start_p7s = 3 + $arg[0]*$interval;
   $p7scroll->coords('p7sb', $start_p7s, 4,
        $start_p7s + $interval*$nodes/$leng, 28);
   if ($bm != 0) {
    $plan7_v->createLine(
        $interval + 0.5*$width, 0.5*$interval - 28,
        $interval + 0.5*$width, 0.4*$interval - 28,
        1.5*$interval + $start_p7s, 0.4*$interval - 28,
        1.5*$interval + $start_p7s, 0.5*$interval - 28,
        -smooth => 0,
        -tags => 'scrollline',
        -joinstyle => 'bevel',
        -arrow => 'last',
        -fill => 'grey',
        -width => 1);
    $plan7_v->createLine(
        3*$interval - 0.5*$width, 0.5*$interval,
        3*$interval - 0.5*$width, 0.6*$interval,
        1.5*$interval + $start_p7s, 0.6*$interval,
         1.5*$interval + $start_p7s, 0.5*$interval,
	-smooth => 0,
        -tags => 'scrollline',
        -joinstyle => 'bevel',
        -arrow => 'first',
        -fill => 'grey',
        -width => 1);
   } # if bm != 0;
  }
}

###########
# Histogram View Button Routines
#
sub Collapse {
    $hist_visible = 0;
    $mmodel_v->delete("box");
    $hist_v->packForget();
    $curr_select = 0;
}


# This subroutine is the callback for switching between
# viewing probabilities and viewing scores in the
# histogram view.

sub update_hist {
  if (!$in_p7) {
    # Must be set same as in mmodel.pl's see_hist to prevent shifting
    $x_coord = ($curr_select -1)*$node_width -3;
    &draw_hist($curr_select, $x_coord);
  } else {
    &see_p7_hist();
  }
}
$end = 1;
